
-- Defines the roles for particular classes and specs.
local _ROLES = {
	["DEATHKNIGHT"] = {"TANK", "DAMAGER", "DAMAGER",},
	["DRUID"] = {"DAMAGER", "DAMAGER", "TANK", "HEALER",},
	["HUNTER"] = "DAMAGER",
	["MAGE"] = "DAMAGER",
	["PALADIN"] = {"HEALER", "TANK", "DAMAGER",},
	["PRIEST"] = {"HEALER", "HEALER", "DAMAGER",},
	["ROGUE"] = "DAMAGER",
	["SHAMAN"] = {"DAMAGER", "DAMAGER", "HEALER",},
	["WARLOCK"] = "DAMAGER",
	["WARRIOR"] = {"DAMAGER", "DAMAGER", "TANK",},
}

-- Determines the best role for a particular class and talent spec.
-- @param class a class name.
-- @param spec_index the specialization index.
-- @return one of "DAMAGER, "HEALER", or "TANK"
local function _GetRoleBySpec(class, spec)
	local roles = _ROLES[class]
	if type(roles) == "table" then
		local role = roles[spec_index]
		if type(role) == "function" then
			return role()
		end
		return role
	end
	return roles
end

-- Determines the best role for the player, based on the player class 
-- and current talent spec.
local function _GetPlayerRoleBySpec()
	local _, playerClass = UnitClass("player")
	local playerSpec = GetSpecialization("player")
	return _GetRoleBySpec(playerClass, playerSpec)
end

-- Sets the player role based on the class and active spec. If the role 
-- is already set to the desired role, no change is made.
local function _SetPlayerRoleBySpec(role)
	role = role or _PlayerGetSpecRole()
	if role == "NONE" then
		return nil
	end
	
	local current_role = UnitGroupRolesAssigned("player")
	if role ~= current_role then
		UnitSetRole("player", role)
	end
	
	return role
end

-- Activates an equipment set with the same name as the active spec.
-- If no such equipment set exists, no action is taken.
local function _SetPlayerGearBySpec()
	local spec_index = GetSpecialization("player")
	local _, spec_name, _, _, _, _, _, _ = GetSpecializationInfo(spec_index)
	UseEquipmentSet(spec_name)
end



-- Construct the addon and the initial state.
local hcRoleBot = CreateFrame("Frame", "hcRoleBotFrame")

hcRoleBot.state = {
	-- Cached role.
	role = nil,
}

--- Fires when group membership information changes.
-- This is used to check when the player joins some sort of group, or 
-- a role change is made for any group member. If the player's role is 
-- changed to a role different than the cached role, the 
-- role is reset to the cached role. If the player is no 
-- longer in a group, the cached role is cleared.
function hcRoleBot:PARTY_MEMBERS_CHANGED(event)
	if not IsInGroup() then
		self.state.role = nil
		return
	end
	
	self.state.role = _SetPlayerRoleBySpec(self.state.role)
end

--- Fires when a player specializations.
-- This is fired for the party and raid members as well as the player. Updates
-- the cached role.
function hcRoleBot:PLAYER_SPECIALIZATION_CHANGED(event, unit)
	if unit != "player" then
		return
	end

	self.state.role = _PlayerSetSpecRole()
	_SetPlayerGearBySpec()
end

--- Fires when a group member's role is changed.
-- If the player role is changed by the player, this will override the 
-- cached role. If any other unit attempts to change the 
-- player's role, it will be reset to the cached role.
--
-- @param changedPlayer the unit changing to a new role.
-- @param changedBy the unit changing the role.
-- @param oldRole the old role.
-- @param newRole the new role.
function hcRoleBot:ROLE_CHANGED_INFORM(event, changedPlayer, changedBy, oldRole, newRole)
	local name, _ = UnitName("player")
	if (target == name) and (changer == name) then
		self.state.role = newRole
	end
end

--- Fires when a role check is started for a group.
-- This suppresses the role popup, which is unnecessary for users of 
-- this addon.
function hcRoleBot:ROLE_POLL_BEGIN(event)
	StaticPopupSpecial_Hide(RolePollPopup)
end

--- Dispatches events to this addon by calling the method with the 
-- same name as the given event.
function hcRoleBot:OnEvent(event, ...)
	if self[event] then
		self[event](self, event, ...)
	end
end

-- Register the event handler and initial events.
hcRoleBot:SetScript("OnEvent", hcRoleBot.OnEvent)

--- Fired when the login process is complete.
-- This registers the events fired when critters should be summoned 
-- or dismissed.
function hcRoleBot:PLAYER_LOGIN(event)
	if not IsLoggedIn() then
		return
	end
	
	self:UnregisterAllEvents()
	
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
	self:RegisterEvent("ROLE_CHANGED_INFORM")
	self:RegisterEvent("ROLE_POLL_BEGIN")
end

hcRoleBot:RegisterEvent("PLAYER_LOGIN")

